//////////////////////////////////////////////////////////////////////////////////
//
//
//   File : sniffer.cpp
//
//   Created : 20-March-01
//
//   Description : This is the packet sniffer that sniffs AntigoneICQ clients chatting
//                 using Antigone 2.0 system.  It prints out messages sent from
//                 AntigoneICQ clients.
//                    
//   Usage:        sniffer <Group Addr> <Group Port>
//
//   *** Change History ***
//
//   20-March-01 Created File : HL
//
//////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <AGroup.h>

// include files for packet sniffer
#include <sys/types.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

// for packet sniffer to join the multicast group.
int mcast_join_group(struct sockaddr_in mcast_addr)
{
  int sd;
  //struct sockaddr_in mgrp;
  struct ip_mreq mreq;
  int on, flags;
  u_char loop;
  sd = socket(PF_INET, SOCK_DGRAM, 0);
  
  
  if (bind(sd, (struct sockaddr *) &mcast_addr, sizeof(struct sockaddr_in)) < 0) {
    perror("bind");
    return (-1);
  }
  
  /* set SO_REUSEADDR on to allow for easier restarts */
  on = 0;
  if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, (char *) &on, sizeof(on)) < 0) {
    perror("reuse");
    exit(1);
  }
  
  memcpy(&mreq.imr_multiaddr, &mcast_addr.sin_addr, sizeof(struct in_addr));
  mreq.imr_interface.s_addr = htonl(INADDR_ANY);
#ifdef DEBUG
  printf("Using %s, port %d\n", inet_ntoa(mcast_addr.sin_addr), ntohs(mcast_addr.sin_port));
#endif
  if (setsockopt(sd, IPPROTO_IP, IP_ADD_MEMBERSHIP, 
                 (char *) &mreq, sizeof(mreq)) < 0) {
    perror("mcast_bind:ADDM");
    return(-1);
  }

  /* set socket to non-blocking */
  flags = fcntl(sd, F_GETFL, 0);
  fcntl(sd, F_SETFL, flags | O_NONBLOCK);
  
  /* turn off loopback */
  loop = 0;
  if (setsockopt(sd, IPPROTO_IP, IP_MULTICAST_LOOP,
      (char *) &loop, sizeof(u_char)) < 0) {
    perror("loopback");
    exit(1);
    }

  return (sd);
}

// for packet sniffer to sniffer packet from the multicast group.
void unicast_recv_action(int mcast_group_socket)
{
  int size;
  char buffer[100];
  struct sockaddr_in unicast_addr;
  socklen_t addr_len = sizeof(struct sockaddr);
  
  size = recvfrom(mcast_group_socket, buffer, 100, 0,
		  (struct sockaddr *)&unicast_addr, &addr_len);
  if (size < 0) {
    if (errno != EWOULDBLOCK) {
      printf("%d",errno);
      cout << "No Sniffer data" << endl;
      return;
    }
  }
  else if (size > 0) {
    // print the data received.
    cout << "Sniffer data is ";
    for(int i=0; i<size; i++)
      cout << buffer[i];
    cout << endl;
  }
}
	
int main (int argc, char* argv[])
{
  if(argc!=3){
    cout << "Usage: sniffer <Group Addr> <Group Port>" << endl;
    exit(1);
  }

  struct sockaddr_in mcast_addr;
  int mcast_group_socket;
  // packet sniffer joining multicast group
  bzero(&mcast_addr, sizeof(struct sockaddr_in));
  mcast_addr.sin_family = AF_INET;
  mcast_addr.sin_addr.s_addr = inet_addr(argv[1]);
  mcast_addr.sin_port = htons((unsigned short)(atoi(argv[2])));
  mcast_group_socket = mcast_join_group(mcast_addr);
  if (mcast_group_socket < 0) {
    cout << "Error calling mcast_join_group() in join_game()" << endl;
    return (-1);
  }
  cout << "Sniffer joined multicast group " << argv[1] << ":" << argv[2] << endl;
  
  int num_fd;
  fd_set rset;
  FD_ZERO(&rset);
  num_fd = 1;
  FD_SET(mcast_group_socket, &rset);
  
  while(true) {
    //select(num_fd, &rset, NULL, NULL, NULL);
    //if(FD_ISSET(mcast_group_socket, &rset))
    unicast_recv_action(mcast_group_socket);
  }
  return 0;
}


